<?php
namespace WMDB\DBFI\Hook;

/**
 * This file is part of the TYPO3 CMS project.
 *
 * It is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License, either version 2
 * of the License, or any later version.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 * The TYPO3 project - inspiring people to share!
 */

use TYPO3\CMS\Backend\Form\DatabaseFileIconsHookInterface;
use TYPO3\CMS\Backend\Form\Element\AbstractFormElement;
use TYPO3\CMS\Backend\Utility\BackendUtility;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Core\Utility\MathUtility;
use TYPO3\CMS\Core\Utility\StringUtility;

/**
 * Class for ['SC_OPTIONS']['t3lib/class.t3lib_tceforms.php']['dbFileIcons'] hook
 */
class DbFileIconsHook implements DatabaseFileIconsHookInterface
{
    /**
     * Modifies the parameters for selector box form-field for the db/file/select elements (multiple)
     *
     * @param array $params An array of additional parameters, eg: "size", "info", "headers" (array with "selector" and "items"), "noBrowser", "thumbnails
     * @param string $selector Alternative selector box.
     * @param string $thumbnails Thumbnail view of images. Only filled if there are images only. This images will be shown under the selectorbox.
     * @param array $icons Defined icons next to the selector box.
     * @param string $rightbox Thumbnail view of images. Only filled if there are other types as images. This images will be shown right next to the selectorbox.
     * @param string $fName Form element name
     * @param array $uidList The array of item-uids. Have a look at \TYPO3\CMS\Backend\Form\FormEngine::dbFileIcons parameter "$itemArray"
     * @param array $additionalParams Array with additional parameters which are be available at method call. Includes $mode, $allowed, $itemArray, $onFocus, $table, $field, $uid.
     * @param AbstractFormElement $parentObject Parent object
     */
    public function dbFileIcons_postProcess(array &$params, &$selector, &$thumbnails, array &$icons, &$rightbox, &$fName, array &$uidList, array $additionalParams, $parentObject) {
        if ($additionalParams['field'] === 'group_db_1') {
            $disabled = '';
            if ($params['readOnly']) {
                $disabled = ' disabled="disabled"';
            }

            $recordUidTableMap = [];
            foreach ($additionalParams['itemArray'] as $item) {
                if (!isset($recordUidTableMap[$item['table']])) {
                    $recordUidTableMap[$item['table']] = array();
                }
                $recordUidTableMap[$item['table']][] = $item['id'];
            }

            $itemArrayCount = count($additionalParams['itemArray']);

            $opt = [];
            foreach ($recordUidTableMap as $table => $uids) {
                $opt[] = '<optgroup label="' . htmlspecialchars($GLOBALS['LANG']->sL($GLOBALS['TCA'][$table]['ctrl']['title'])) . '">';
                if (is_array($uids)) {
                    for ($i = 0, $count = count($uids); $i < $count; ++$i) {
                        $uid = $uids[$i];
                        $pRec = BackendUtility::getRecordWSOL($table, $uid);
                        if (is_array($pRec)) {
                            $pTitle = BackendUtility::getRecordTitle($table, $pRec, false, true);
                            $pUid = $table . '_' . $uid;
                            $opt[] .= '<option value="' . $pUid . '" title="' . $pTitle . '">' . $pTitle . '</option>';
                        }
                    }
                }
                $opt[] .= '</optgroup>';
            }

            // Create selector box of the options
            $sSize = $params['autoSizeMax']
                ? MathUtility::forceIntegerInRange($itemArrayCount + 1, MathUtility::forceIntegerInRange($params['size'], 1), $params['autoSizeMax'])
                : $params['size'];
            $isMultiple = $params['maxitems'] != 1 && $params['size'] != 1;
            $selector = '<select id="' . StringUtility::getUniqueId('tceforms-multiselect-') . '" '
                . ($params['noList'] ? 'style="display: none"' : 'size="' . $sSize . '" class="form-control tceforms-multiselect"')
                . ($isMultiple ? ' multiple="multiple"' : '')
                . ' data-formengine-input-name="' . htmlspecialchars($fName) . '" ' . $this->getValidationDataAsDataAttribute($additionalParams['config']) . $additionalParams['onFocus'] . $params['style'] . $disabled . '>' . implode('',
                    $opt)
                . '</select>';
        }
    }

    /**
     * Build JSON string for validations rules and return it as data attribute for HTML elements.
     * It's the same method as in AbstractFormElement.
     *
     * @param array $config
     * @return string
     */
    protected function getValidationDataAsDataAttribute(array $config)
    {
        return sprintf(' data-formengine-validation-rules="%s" ',
            htmlspecialchars($this->getValidationDataAsJsonString($config)));
    }

    /**
     * Build JSON string for validations rules.
     * It's the same method as in AbstractFormElement.
     *
     * @param array $config
     * @return string
     */
    protected function getValidationDataAsJsonString(array $config)
    {
        $validationRules = array();
        if (!empty($config['eval'])) {
            $evalList = GeneralUtility::trimExplode(',', $config['eval'], true);
            unset($config['eval']);
            foreach ($evalList as $evalType) {
                $validationRules[] = array(
                    'type' => $evalType,
                    'config' => $config
                );
            }
        }
        if (!empty($config['range'])) {
            $validationRules[] = array(
                'type' => 'range',
                'config' => $config['range']
            );
        }
        if (!empty($config['maxitems']) || !empty($config['minitems'])) {
            $minItems = (isset($config['minitems'])) ? (int)$config['minitems'] : 0;
            $maxItems = (isset($config['maxitems'])) ? (int)$config['maxitems'] : 10000;
            $type = ($config['type']) ?: 'range';
            if ($config['renderType'] !== 'selectTree' && $maxItems <= 1 && $minItems > 0) {
                $validationRules[] = array(
                    'type' => $type,
                    'minItems' => 1,
                    'maxItems' => 100000
                );
            } else {
                $validationRules[] = array(
                    'type' => $type,
                    'minItems' => $minItems,
                    'maxItems' => $maxItems
                );
            }
        }
        if (!empty($config['required'])) {
            $validationRules[] = array('type' => 'required');
        }
        return json_encode($validationRules);
    }
}